import { Component, Input, AfterViewInit, OnChanges, OnDestroy } from '@angular/core';
import {    lightningChart, 
            ChartXY, 
            Point, 
            ColorRGBA, 
            FontSettings, 
            Themes, 
            UIOrigins, 
            UIDraggingModes, 
            emptyFill, 
            emptyLine, 
            SolidFill, 
            SolidLine, 
            Animator, 
            AnimationEasings 
        } from '@arction/lcjs';

@Component({
  selector: 'app-chart',
  template: '<div [id]="this.chartId"></div>',
  styles: ['div { height: 100% }']
})

export class ChartComponent implements OnChanges, OnDestroy, AfterViewInit {
  chart!: ChartXY;
  chartId!: number;

  constructor() {}

  ngOnChanges() {
  }

ngAfterViewInit() {

        const fonts = {
            subTitle:new FontSettings({
                size: 15,
            }),
                title: new FontSettings({
                size: 40,
                weight: 400
            })
        }

        this.chart = lightningChart()
        .ChartXY({container: `${this.chartId}`, theme:Themes.auroraBorealisNew})



        this.chart.addUIElement(undefined, this.chart.uiScale)
        .setPosition({ x: 50, y: 93 })
        .setOrigin(UIOrigins.Center)
        .setText('- With Bubbles -')
        .setTextFont(fonts.subTitle)
        .setDraggingMode(UIDraggingModes.notDraggable)
        .setBackground((bg) => bg
            .setFillStyle(emptyFill)
            .setStrokeStyle(emptyLine)
        )

        const bubbleWidthHeightRatio = {
            x: window.innerHeight / window.innerWidth,
            y: 1
        }

        // Create instance of ellipse series to draw bubbles.
        const ellipseSeries = this.chart.addEllipseSeries()
        let bubbleCount = 0

        // Handler of dragging bubbles.
        const bubbleDragHandler = (figure: { getDimensions: () => any; setDimensions: (arg0: any) => void; scale: { x: { getPixelSize: () => number; }; y: { getPixelSize: () => number; }; }; }, event: any, button: any, startLocation: any, delta: { x: number; y: number; }) => {
            const prevDimensions = figure.getDimensions()
            figure.setDimensions(Object.assign(prevDimensions, {
                x: prevDimensions.x + delta.x * figure.scale.x.getPixelSize(),
                y: prevDimensions.y + delta.y * figure.scale.y.getPixelSize()
            }))
        }


        // Create resizeBubble array and sizeArray to store the values separately 
        const resizeBubble: string | any[] = []
        const sizeArray: number[] = []

        // Create a single bubble to visualize in specific coordinates and specified size.
        const addBubble = (pos: { x: any; y: any; }, size: number) => {
            const radius = size * 2.5

            var num = Math.round(0xffffff * Math.random());
            var r = num >> 16;
            var g = num >> 8 & 255;
            var b = num & 255;

            const fillStyle = new SolidFill({ color: ColorRGBA( r, g, b, 95 ) });
            const strokeStyle = new SolidLine({  fillStyle: new SolidFill({ color: ColorRGBA( r, g, b, 95 ) }), thickness: 1 })

            const figure = ellipseSeries.add({
                x: pos.x,
                y: pos.y,
                radiusX: radius * bubbleWidthHeightRatio.x,
                radiusY: radius * bubbleWidthHeightRatio.y
            })
                .setFillStyle(fillStyle)
                .setStrokeStyle(strokeStyle)

            // Make draggable by mouse.
            figure.onMouseDrag(bubbleDragHandler)
            bubbleCount++
            return figure
        }

        const addRandomBubble = () => {
            const pos = {
                x: Math.random() * 200 - 100,
                y: Math.random() * 200 - 100
            }
            const size = 1 + Math.random() * 7.0
            sizeArray.push(size)
            resizeBubble.push(addBubble(pos, size))
        }

        const bubbleMaxCount = 100

        // Animate bubbles creation.
        Animator(() => undefined)(2.5 * 1000, AnimationEasings.ease)([[0, bubbleMaxCount]], ([nextBubbleCount]) => {
            while (bubbleCount < nextBubbleCount)
                addRandomBubble()
        })
}

ngOnDestroy() {
        // "dispose" should be called when the component is unmounted to free all the resources used by the chart
        this.chart.dispose();
      }
}